/*========================================================================================
// Project: Interface with COMAX II Laser/Receiver Unit.
// Author:  Jim Abraham
// Date:    3-3-20
// Version: V1.0
// Board: Arduino Mega 2560
// Revisions: 
//========================================================================================
*/
#include <EEPROM.h> // for storing calibration parameters
#include <SPI.h>    // for DAC
#include "MgsModbus.h"  // modbus TCP IP interface
#include <genieArduino.h>
Genie genie;
#define RESETLINE 4 // LCD display reset

//---------------------------------------------------------------------------------------
// Variables used by ISR.
//---------------------------------------------------------------------------------------
volatile int state1; 
volatile unsigned int tst, tend, tst_indexbegin,tst_indexend,tst_beambegin,tst_beamend;
volatile int index_pulse_start=0,index_pulse_end=0;
//---------------------------------------------------------------------------------------
unsigned int timer1_counter;  //integer to store the value of timer
float timer1_counter_divide=3.246; //3.249
unsigned int timer1_counter_index,timer1_counter_beam;
int pulse_width_cnt_update=0;

// Arrays are zero indexed, that is, referring to the array initialization below,
// the first element of the array is at index 0.
int stack_count=0;
int pulse_cnts[990];  // 990 = max stack size.
int beam_counts[64];  // average 64 readings for 1 stack count.

// beam and index pulses
long beam_counts_stack=0;
int beam_count=0,i=0;
int beam_counter=0;
int beam_count_ready=1;
int stack_full_flag=0;
int beam_count_avg=0, beam_count_avg1=0,beam_count_reading=0;

// user entry
int counts_sp, counts_sp_new;
int beam_count_sp=8;
int zero_pb=0,span_pb=0;

float glass_level_reading_out=0;

// Alarming Variables
volatile int index_pulse_ok=0,beam_pulse_ok=0;
int LOI_led=0, LOB_led=0;
int LHA_led=0,LLA_led=0;
int LHT_led=0,LLT_led=0;
int RHT_led=0,RLT_led=0;
int LOI_flag=0,LOI_bad=0,LOB_flag=0,LOB_bad=0;
int LHA_flag=0,LHA_bad=0,LLA_flag=0,LLA_bad=0;
int LHT_flag=0,LHT_bad=0,LLT_flag=0,LLT_bad=0;
int RHT_flag=0,RHT_bad=0,RLT_flag=0,RLT_bad=0;
int Laser_Temp_alarm_HLIM=140,Laser_Temp_alarm_LLIM=60;
int Rec_Temp_alarm_HLIM=140,Rec_Temp_alarm_LLIM=60;
int Laser_Temp_reading,Rec_Temp_reading;
int Alarm_buzzer=12,Alarm_light=13;
float glevel_reading_alarm_LLIM=-0.100,glevel_reading_alarm_HLIM=0.100;

// populate during calibration mode.
float ZCNT,ZCNT_LVL,ZCNT_LVL_4D; //float ZCNT=4000,ZCNT_LVL=0.0,ZCNT_LVL_4D=0;
float ZCNT_new,ZCNT_LVL_new;
float SCNT,SCNT_LVL,SCNT_LVL_4D;
float SCNT_new,SCNT_LVL_new;
int ZCNT_LVL_4D_new,SCNT_LVL_4D_new;
int val,val1,val0,val11,beam_offset=1500;
int trend_cnt=0;

// the current address in the EEPROM (i.e. which byte we're going to write to next)
int addr = 0;

// calculate glass level readings.
float glass_level_reading,glass_level_calc1,glass_level_calc2;
int glass_level_reading_4D;
int gl_sign=0,wdt_led=0,rebuild_led=0;

// delay for check LCD inputs.
unsigned long waitPeriod = 500;
unsigned long lastRun = 0;

// Simulate Beam counts for testing. line 183.
char input;
int input_int=5500;
String serialData;

const int PIN_CS = 53; // for DAC
unsigned int DAC_count;

// modbus variables
MgsModbus Mb;
char inByte;     // incoming serial byte (characters)
char chr[10];
String inByte1;  // incoming serial byte (string)
String inByte2;  // incoming serial byte (string)
int charcnt=0,charcnt1=0,charflag=0;
//---------------------------------------------------------------------------
// Ethernet settings (depending on MAC and Local network)
//---------------------------------------------------------------------------
  byte mac[] = {0xA8, 0x61, 0x0A, 0xAE, 0x3A, 0xFC }; // get from ethernet shield.
//  byte mac[] = {0x90, 0xA2, 0xDA, 0x0E, 0x94, 0xB5 };  
  IPAddress ip(10, 53, 77, 247);          // Tank 1 VLAN
  IPAddress gateway(10, 53, 77, 254);
  IPAddress subnet(255, 255, 255, 192);
//---------------------------------------------------------------------------

//=======================================================================================
// SET-UP SECTION
//=======================================================================================
void setup()
{
//---------------------------------------------------------------------------------------
// check positive/negative pulse edges and record clock pulses.
// beam count = start: negiatve index pulse, end: positive beam pulse.
// index count = start: negiatve index pulse, end: positive index pulse.
//---------------------------------------------------------------------------------------
    attachInterrupt(digitalPinToInterrupt(2), index_pulse, CHANGE);  //pin2 interrupt Index start
    attachInterrupt(digitalPinToInterrupt(3), beam_pulse, CHANGE);   //pin3 interrupt Beam end

    pinMode(12, OUTPUT);    // sets the digital pin 12 as alarm buzzer output
    pinMode(13, OUTPUT);    // sets the digital pin 13 as output
    pinMode(PIN_CS, OUTPUT);    
  
// get parameters from EEPROM.
//  Mega_EEPROM_Write_CAL_DATA();
//  Mega_EEPROM_Write_Stack_Cnt();
    Mega_EEPROM_Read();

    SPI.begin();
    SPI.setClockDivider(SPI_CLOCK_DIV2);

//---------------------------------------------------------------------------------------
// Set-Up Timer 1. 
// Used to count 16MHz clocks to generate counts above.
//---------------------------------------------------------------------------------------
   TCCR1A = 0;  //Timer 1 in normal mode.
   TCCR1B = 1;  //|= (1<<CS11);//start the Timer 1 with no prescaler, 16MHz clock pulses
   TCNT1 = 0;   //making intial Timer 1 counter to zero 
//---------------------------------------------------------------------------------------
  Serial.begin(9600);     // Debug Communications.
  Serial1.begin(115200);  // 4D TouchScreen Communications.
  genie.Begin(Serial1);   // Use Serial1 for talking to the Genie Library, and to the 4D Systems display
  genie.AttachEventHandler(myGenieEventHandler); // Attach the user function Event Handler for processing events
  pinMode(RESETLINE, OUTPUT); // Set D4 on Arduino to Output (4D Arduino Adaptor V2 - Display Reset)
  digitalWrite(RESETLINE, 0); // Reset the Display via D4, Display Reset is Active Low
  delay(200);
  digitalWrite(RESETLINE, 1); // unReset the Display via D4
  delay (5000); //let the display start up after the reset (This is important)
  genie.WriteContrast(11); // use 0-15 for Brightness Control, where 0 = Display OFF, though to 15 = Max Brightness ON.
  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x00, counts_sp);         // FORM2 stack count set-pt
  genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x01, ZCNT);           // FORM1
  genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x04, ZCNT_LVL_4D);    // FORM1 
  genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x02, SCNT);           // FORM1
  genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x03, SCNT_LVL_4D);    // FORM1 
  state1=0;
  
//----------------------------------------------------------------  
// initialize the ethernet device
//----------------------------------------------------------------  
  Ethernet.begin(mac, ip, gateway, subnet);   // start ethernet interface
//  Mb.MbData[0] = 0,  Mb.MbData[1] = 0,  Mb.MbData[2] = 0,  Mb.MbData[3] = 0;
//  Mb.MbData[4] = 0,  Mb.MbData[5] = 0,  Mb.MbData[6] = 0,  Mb.MbData[7] = 0;
//  Mb.MbData[8] = 0,  Mb.MbData[9] = 0,  Mb.MbData[10] = 0, Mb.MbData[11] = 0;
//----------------------------------------------------------------  
}//end of setup

//=======================================================================================
// Interrupt Sub-Routines
//=======================================================================================
void index_pulse()
//---------------------------------------------------------------------------------------
{
  index_pulse_ok=1;  // alarming.
  if(digitalRead(2)==LOW)
    {   
      if (state1==0)
        { 
          tst_indexbegin = TCNT1; // Get starting count. calc pulse count.
       //   index_pulse_start=1; // for calc pulse count.
          tst = TCNT1;  // Get starting count. for beam count.
          state1=1;     // change flag to get end count. (end of beam pulse, pos edge)
  //        index_pulse_ok=1;  // alarming.        
        }
    }
  else
    {
// if(digitalRead(2)==HIGH) // for index pulse width counts.
//    {   
    if (state1==2 && tst_indexend==0)
      {
        tst_indexend = TCNT1;  // Get ending count.
    //    TCNT1=0;
   //     index_pulse_end=1;
      }
    }
}
//---------------------------------------------------------------------------------------
void beam_pulse()
//---------------------------------------------------------------------------------------
{
  beam_pulse_ok=1; // for alarming
 if(digitalRead(3)==HIGH)
    { 
       if (state1==1)  //beam pulse high.
        {
          tend = TCNT1;    // Get end count.
          tst_beamend= TCNT1;    // Get beginning count.
          state1=2;        // change flag to get calc total counts.
     //     beam_pulse_ok=1; // for alarming
        }
   }
 else
   {
     if (state1==1 && tst_beambegin==0)
       {
          tst_beambegin= TCNT1;    // Get beginning count.(low)
       }
   }
}

//=======================================================================================
// Regular Routines
//=======================================================================================
//---------------------------------------------------------------------------------------
void Calc_beam_count()  // get readings, avg them and then put in stack.
//---------------------------------------------------------------------------------------
{
  if (timer1_counter> 200 && beam_count_ready==0)
    {
      timer1_counter = timer1_counter/timer1_counter_divide;//3.249; // 16MHz pulses, make 5MHz.
      beam_counts[beam_count]=timer1_counter-beam_offset; // input_int-beam_offset;// 64 elements in stack.
      beam_count++;
      if (beam_count>=beam_count_sp)  //stack rebuilt (takes about 2 min)
        {
          for (i = 0 ; i < beam_count_sp ; i++){beam_counts_stack += beam_counts[i];}
          beam_count_reading = beam_counts_stack / beam_count_sp;
          beam_counts_stack=0;
          beam_count_ready=1;
          beam_count=0;
        }  
    }
    
}

//---------------------------------------------------------------------------------------
void Populate_Stack()
//---------------------------------------------------------------------------------------
{
  if (beam_count_ready==1)
    {
      beam_count_ready=0;
      pulse_cnts[stack_count] = beam_count_reading; // populate stack
      stack_count++;
      if (stack_full_flag==0)
      {
        genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x06, stack_count); //FORM2 stack count   
        genie.WriteObject(GENIE_OBJ_LED, 0x00, rebuild_led); //FORM2 rebuild stack led 
        genie.WriteObject(GENIE_OBJ_LED, 0x01, rebuild_led); //FORM0 rebuild stack led 
        genie.WriteObject(GENIE_OBJ_LED, 0x02, rebuild_led); //FORM1 rebuild stack led 
        genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x07,beam_count_reading);       // FORM3
        rebuild_led^=1;
      }
        
      if (stack_count > counts_sp-1 && stack_full_flag==0)  //0-249 = 250 counts.
        {
          stack_full_flag=1; // stack full of radings.
          zero_pb=0;
          span_pb=0;
          stack_count=0;
          for (i = 0 ; i < counts_sp ; i++){beam_counts_stack += pulse_cnts[i];}
          beam_count_avg = beam_counts_stack / counts_sp;
          beam_counts_stack=0;
        }
      else if (stack_full_flag==1)  // stack full, just replacing readings...
        {
          genie.WriteObject(GENIE_OBJ_LED, 0x00,0); //FORM2 rebuild stack led 
          genie.WriteObject(GENIE_OBJ_LED, 0x01,0); //FORM0 rebuild stack led 
          genie.WriteObject(GENIE_OBJ_LED, 0x02,0); //FORM1 rebuild stack led 
          beam_count_sp=8;//8;  // stack rebuilt, take more samples per reading.
          if (stack_count > counts_sp-1){stack_count=0;}
          for (i = 0 ; i < counts_sp ; i++){beam_counts_stack += pulse_cnts[i];}
          beam_count_avg = beam_counts_stack / counts_sp;
          beam_counts_stack=0;      
        }
    }
}

//---------------------------------------------------------------------------------------
void Calc_Glass_Level()
//---------------------------------------------------------------------------------------
{
  if (stack_full_flag==1)
    {
      //--------------------------------------------------------------------------------
      // Calculate Glass Level Readings
      // float ZCNT=4000 = level of 0.000"
      // float SCNT=2500 = level of 0.250"  
      // float ZCNT_LVL=0.000,SCNT_LVL=0.250;
      //--------------------------------------------------------------------------------
      beam_count_avg1 = beam_count_avg;// - 1500;    // make 4000 counts represent 0" level.
      glass_level_calc1 = (SCNT_LVL-ZCNT_LVL) / (SCNT-ZCNT);      
      glass_level_calc2 = beam_count_avg1 - ZCNT;                      //bot
      glass_level_reading= glass_level_calc1 * glass_level_calc2;
     //--------------------------------------------------------------------------------
     // 4D LCD
     //--------------------------------------------------------------------------------
      glass_level_reading_4D=abs(glass_level_reading * 1000);
      if (glass_level_reading < 0){gl_sign=1;}      // negative reading
      else{gl_sign=0;}                              // positive reading
     //--------------------------------------------------------------------------------
      int glass_level_reading_Scope= glass_level_reading * 1000;
      if (glass_level_reading_Scope > 100){glass_level_reading_Scope=100;}
      if (glass_level_reading_Scope < -100){glass_level_reading_Scope=-100;}
      trend_cnt++;
      if (trend_cnt > 10) // delay trend update. (total trend update is 5 min)
        {
          //chart (-100" to 100")
          genie.WriteObject(GENIE_OBJ_SCOPE, 0x00, glass_level_reading_Scope);    // FORM3 
          genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x00,glass_level_reading_4D);// FORM3
          genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x05,beam_count_avg1);       // FORM3
          genie.WriteObject(GENIE_OBJ_USER_LED, 0x0B, gl_sign);                   // FORM3
          genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x08,beam_count_avg1);       // FORM3
          genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x09,stack_count);       // FORM3
          trend_cnt=0;
        }
     //--------------------------------------------------------------------------------
    }  
}

//=======================================================================================
// MAIN LOOP
//=======================================================================================
void loop()
{
//-------------------------------------------------------------------------------
// This calls the library each loop to process the queued responses from the display  
//-------------------------------------------------------------------------------
  genie.DoEvents();
  if (millis() - lastRun >= waitPeriod) 
  {
  // The results of this call will be available to myGenieEventHandler() after the display has responded
  // Do a manual read
    genie.ReadObject(GENIE_OBJ_WINBUTTON, 0x06);  // FORM2 add 10 pb
    genie.ReadObject(GENIE_OBJ_WINBUTTON, 0x09);  // FORM2 sub 10 pb
    genie.ReadObject(GENIE_OBJ_WINBUTTON, 0x0A);  // FORM2 enter pb
    lastRun = millis();
  }
  
//-------------------------------------------------------------------------------
if (state1==3)
  {
      delay(40);
      timer1_counter = tend - tst; // Calculate total counts.
      Calc_beam_count();           // Get Beam count from index/beam pulses.
      Populate_Stack();            // Put new reading into stack.
      Calc_Glass_Level();          // Calculate the new glass level reading.
      
      pulse_width_cnt_update++;
  if (pulse_width_cnt_update >= 20)
      {
        Update_Analog_Output();     // Update 4-20ma output to DCS/PLC.
        genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x0A, timer1_counter_index);  // FORM4
        genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x0B, timer1_counter_beam);   // FORM4
        pulse_width_cnt_update=0;
      }  
    tst_indexbegin=0;
    tst_indexend=0;
    tst_beambegin=0;
    tst_beamend=0;    
    state1=0;   // start looking for beginning of index pulse again.
    TCNT1 = 0;
  }

//----------------------------------------------------------------
// laser/reciever unit pulses to control box
//----------------------------------------------------------------
// 250usec = 0.25 msec
// index pulse = 1600usec
// beam pulse = 600usec
//----------------------------------------------------------------
  if (tst_indexend>0 && state1==2) //calculate index pulse count, should be around 8000.
    {
      timer1_counter_index = tst_indexend - tst_indexbegin; // Calculate total counts for index pulse.
      timer1_counter_index = timer1_counter_index/timer1_counter_divide;//3.249;

      timer1_counter_beam = tst_beamend - tst_beambegin; // Calculate total counts for index pulse.
      timer1_counter_beam = timer1_counter_beam/timer1_counter_divide;//3.249;
      state1=3;  // Reset state flag.
    }

//-------------------------------------------------------------------------------
// Update local TouchScreen LCD display.
//-------------------------------------------------------------------------------
  Update_FORMS();
  
}// main loop
//=======================================================================================

//-------------------------------------------------------------------------------
// Write Data to Mega 2560 EEPROM
//-------------------------------------------------------------------------------
// 1. Stack Count (0-999)
// 2. ZCNT        (0-8000)
// 3. ZCNT level  (0.000)
// 4. SCNT        (0-8000)
// 5. SCNT level  (0.250)

//-------------------------------------------------------------------
void Mega_EEPROM_Write_Stack_Cnt()
//-------------------------------------------------------------------
{
// 1. Stack Count (0-999)
// 256 counts
// 256 x 0.10 = 25
// val=25
// 25 x 10 = 250 so 256 - 250 = 6.  
// val1=6   
  val = counts_sp * 0.1;
  val1 = counts_sp - val*10;
  EEPROM.write(0, val);   // counts_sp 25
  EEPROM.write(1, val1);  // counts_sp 6
  delay(100);
}

//-------------------------------------------------------------------
void Mega_EEPROM_Write_CAL_DATA()
//-------------------------------------------------------------------
{
// Zero Calibration parameters
  if (zero_pb ==1)
  {
    val = ZCNT * 0.01;
    val1 = ZCNT - val*100;
    EEPROM.write(2, val);   // ZCNT MS
    EEPROM.write(3, val1);  // ZCNT LS
    val0 = ZCNT_LVL * 1000;    // 1001
    val = val0 * 0.01;         // 10
    val11 = val * 100;         // 1000 
    val1 = val0 - val11;       // 01
    EEPROM.write(4, val);   // ZCNT_LVL 10
    EEPROM.write(5, val1);  // ZCNT_LVL 01
  }

// Span Calibration parameters
  if (span_pb ==1)
    {
      val = SCNT * 0.01;
      val1 = SCNT - val*100;
      EEPROM.write(6, val);   // SCNT 40
      EEPROM.write(7, val1);  // SCNT 23
      val0 = SCNT_LVL * 1000;    // 250
     // if (val0 < 1000) {val0 = val0 * 10;}    // 2500
      val = val0 * 0.01;         // 25
      val11 = val * 100;         // 2500 
      val1 = val0 - val11;       // 00
      EEPROM.write(8, val);   // SCNT_LVL 25
      EEPROM.write(9, val1);  // SCNT_LVL 00
    }
  delay(100);
}

//-------------------------------------------------------------------------------
// Read Data to Mega 2560 EEPROM
//-------------------------------------------------------------------------------
void Mega_EEPROM_Read()
//-------------------------------------------------------------------------------
{
    counts_sp = EEPROM.read(0)*10 + EEPROM.read(1);
    counts_sp_new = counts_sp;
    
    ZCNT = EEPROM.read(2)*100 + EEPROM.read(3);
    ZCNT_LVL = EEPROM.read(4)*100 + EEPROM.read(5);
    ZCNT_LVL_4D = ZCNT_LVL;
    ZCNT_LVL = ZCNT_LVL /10000; // 0.00
    
    SCNT = EEPROM.read(6)*100 + EEPROM.read(7);
    SCNT_LVL = EEPROM.read(8)*100 + EEPROM.read(9);
    SCNT_LVL_4D = SCNT_LVL;
    SCNT_LVL = SCNT_LVL /10000; //=2500/10000 = 0.25
    delay(500);  
}

//=======================================================================================
// Check for alarms, Turn on alarm output if needed and update Form0
//=======================================================================================
void Update_Analog_Output()
{
// analogRead values go from 0 to 1023, analogWrite values from 0 to 255 (only 8 bit)
// output glass lelev reading to the Tank DCS.
// use SPI to MC4688 micro chip. 12bit 0-5v
//-------------------------------------------------------------------------------
// ma 4   8   12   16   20   4-20ma
//  v 1   2    3    4    5   1-5 vdc
//    0%      50%        100%
//   -0.25              +0.25
// y = level, V=output voltage
// level = +/- 0.25",  voltage = 1-5vdc dac counts = 0-4095
//y = c1 x V) + c2
//V = (y - c2) / c1
// DAC count (Output value) = Desired output voltage / Full-scale voltage * 4096
//-------------------------------------------------------------------------------
  if (LOI_bad==0 && LOB_bad==0 && stack_full_flag==1)
    {
      float c1=0.125, c2=-0.375;
      float volts = (glass_level_reading - c2) / c1;
      DAC_count = (volts / 5) * 4096;
      set_Volt_Output(DAC_count); // set voltage output.
      Send_Data_to_PI();          // Send data to PI.
// Serial.println(volts);
// Serial.println(DAC_count);
// Serial.println(glass_level_reading); 
    }
}

//-------------------------------------------------------------------------------
//Set voltage output to 4-20ma module. (assuming single channel, gain=2)
//-------------------------------------------------------------------------------
void set_Volt_Output(unsigned int val)
{
  byte lowByte = val & 0xff;
  byte highByte = ((val >> 8) & 0xff) | 0x10;
  digitalWrite(PIN_CS,LOW);
  SPI.transfer(highByte);
  SPI.transfer(lowByte);
  digitalWrite(PIN_CS,HIGH);
}

//---------------------------------------------------------------------------
// Batch Moisture data to send to PI.
//---------------------------------------------------------------------------
 void Send_Data_to_PI()
 {
   Mb.MbData[0] = glass_level_reading_4D;  // make an integer to send, /100 in PI.
   Mb.MbsRun(); // need to run as often as possible...otherwise get i/o timeout.
 }
 
//=======================================================================================
// Check for alarms, Turn on alarm output if needed and update Forms
//=======================================================================================
void Check_Alarms()
{
  Check_Pulses();
  Check_Level_Height();
//  Check_Laser_Temp();
//  Check_Reciever_Temp();
  Check_Sound_Alarm();
}

//-------------------------------------------------------------------------------
void Check_Pulses()
//-------------------------------------------------------------------------------
{
//loss of index pulse alarm
  if (index_pulse_ok==0)
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x02, LOI_led);  // FORM0 loss of index led 
      LOI_led^=1;
      // set flag, re-populate stack after good again.
      LOI_flag++;
      if (LOI_flag >= 30)
        {
          LOI_bad=1;    // if 30 readings bad, repopulate stack if trying to calibate.
          LOI_flag=30;
        }
    }
  else
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x02, 0);  // FORM0 loss of index led 
      LOI_flag=0;
      LOI_bad=0;
      // set good flag
    }
    
//loss of beam pulse alarm
  if (beam_pulse_ok==0)
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x07, LOB_led);  // FORM0 loss of beam led 
      LOB_led^=1;
      // set flag, re-populate stack after good again.
      LOB_flag++;
      if (LOB_flag >= 30)
        {
          LOB_bad=1;    // if 30 readings bad, repopulate stack if trying to calibate.
          LOB_flag=30;
        }    
    }
  else
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x07, 0);  // FORM0 loss of beam led 
      // set good flag
      LOB_flag=0;
      LOB_bad=0;
    }
  
// if bad flag set and both good flags set then, repopulate stack. 
  if (LOI_bad==1 || LOB_bad==1)
    {
      ////beam_count_sp=1;
      beam_count_ready=0;
      beam_count=0;
      stack_full_flag=0;  // stack full of readings.
      stack_count=0;
    }

  index_pulse_ok=0;
  beam_pulse_ok=0;  
}

//-------------------------------------------------------------------------------
void Check_Level_Height()
//-------------------------------------------------------------------------------
{
//level high alarm
  if (glass_level_reading >= glevel_reading_alarm_HLIM)// && gl_sign==0)
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x03, LHA_led);   // FORM0 high level led 
      LHA_led^=1;
      LHA_bad=1;
    }
  else
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x03, 0);         // FORM0 high level led 
      LHA_bad=0;
    }
//level low alarm
  if (glass_level_reading <= glevel_reading_alarm_LLIM)// && gl_sign==1)
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x08, LLA_led);  // FORM0 low level led 
      LLA_led^=1;
      LLA_bad=1;
    }
  else
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x08, 0);       // FORM0 low level led  
      LLA_bad=0;
    }
}

//-------------------------------------------------------------------------------
void Check_Laser_Temp()
//-------------------------------------------------------------------------------
{
//Laser Temperature high alarm
  if (Laser_Temp_reading >= Laser_Temp_alarm_HLIM)
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x04, LHT_led);   // FORM0 high temp led 
      LHT_led^=1;
      LHT_bad=1;
    }
  else
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x04, 0);         // FORM0 high temp led 
      LHT_bad=0;
    }
//Laser Temperature low alarm
  if (Laser_Temp_reading <= Laser_Temp_alarm_LLIM)
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x09, LLT_led);  // FORM0 low temp led 
      LLT_led^=1;
      LLT_bad=1;
    }
  else
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x09, 0);       // FORM0 low temp led  
      LLT_bad=0;
    }
}

//-------------------------------------------------------------------------------
void Check_Reciever_Temp()
//-------------------------------------------------------------------------------
{
//Reciever Temperature high alarm
  if (Rec_Temp_reading >= Rec_Temp_alarm_HLIM)
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x05, RHT_led);   // FORM0 high temp led 
      RHT_led^=1;
      RHT_bad=1;
    }
  else
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x05, 0);         // FORM0 high temp led 
      RHT_bad=0;
    }
//Laser Temperature low alarm
  if (Rec_Temp_reading <= Rec_Temp_alarm_LLIM)
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x06, RLT_led);  // FORM0 low temp led 
      RLT_led^=1;
      RLT_bad=1;
    }
  else
    {
      genie.WriteObject(GENIE_OBJ_USER_LED, 0x06, 0);       // FORM0 low temp led  
      RLT_bad=0;
    }
}

//-------------------------------------------------------------------------------
void Check_Sound_Alarm()
//-------------------------------------------------------------------------------
{
   if (LOI_bad==1 || LOB_bad==1 || LHA_bad==1 || LLA_bad==1 || LHT_bad==1 || LLT_bad==1 || RHT_bad==1 || RLT_bad==1)
    {
      digitalWrite(Alarm_buzzer, HIGH);   // sets the digital pin 12 on (Alarm Buzzer)
      digitalWrite(Alarm_light, HIGH);    // sets the digital pin 13 on (Alarm Light)
      //delay(100);                         // waits for a second
     // digitalWrite(Alarm_light, LOW);     // sets the digital pin 13 off
     // delay(100);   
    }
  else
    {
      digitalWrite(Alarm_buzzer, LOW);   // sets the digital pin 12 off
      digitalWrite(Alarm_light, LOW);    // sets the digital pin 13 off
    }
}

//=======================================================================================
// This is the user's event handler. It is called by genieDoEvents()
// when the following conditions are true
// The link is in an IDLE state, and there is an event to handle
//=======================================================================================
void myGenieEventHandler(void)
{
  genieFrame Event;
  genie.DequeueEvent(&Event);

  // If the cmd received is from a Reported Event (Events triggered from the 
  //Events tab of Workshop4 objects)
  if (Event.reportObject.cmd == GENIE_REPORT_EVENT)
  {
    if (Event.reportObject.object == GENIE_OBJ_WINBUTTON) // Reported Message from a PB
    {
      //------------------------------------------------------------------
      // Add 10 Counts to Count set-pt.
      //------------------------------------------------------------------
      if (Event.reportObject.index == 6)    //FORM2                             
      {
        counts_sp_new = counts_sp_new + 10;
        genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x00, counts_sp_new);         
      }
      //------------------------------------------------------------------
      // Subtract 10 Counts to Count set-pt.
      //------------------------------------------------------------------
      if (Event.reportObject.index == 9)    //FORM2                                
      { 
        counts_sp_new = counts_sp_new - 10;
        genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x00, counts_sp_new);  
      }
      //------------------------------------------------------------------
      // Enter new Count set-pt. (Save Stack Count)
      //------------------------------------------------------------------
      if (Event.reportObject.index == 10)   //FORM2                                 
      {
        if (counts_sp != counts_sp_new)
          {
            counts_sp = counts_sp_new;
            Mega_EEPROM_Write_Stack_Cnt();
            // rebuild stack
            beam_count_ready=0;
            beam_count=0;
            stack_full_flag=0;  // stack full of readings.
            stack_count=0;
          }
       }
      //------------------------------------------------------------------
      // Check if ZERO PB pressed.
      //------------------------------------------------------------------
      if (Event.reportObject.index == 2 && stack_full_flag==1)    //FORM1                               
        {
          zero_pb++;
          if (zero_pb == 1)
            {
              genie.WriteObject(GENIE_OBJ_LED, 0x04, 1); //zero is set led
              ZCNT_new = beam_count_avg1; // ZERO = 5500-1500 = 4000 ***
              ZCNT_LVL_new = glass_level_reading;
              ZCNT_LVL_4D_new = abs(glass_level_reading * 1000);
              genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x01, ZCNT_new);
              genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x04, ZCNT_LVL_4D_new);
            }       
          else
            {
              if (zero_pb >1)
                {
                  zero_pb=0;
                  genie.WriteObject(GENIE_OBJ_LED, 0x04, 0); //zero is set led
                  genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x01, ZCNT);
                  genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x04, ZCNT_LVL_4D);
                }  
            }
        }
        
       //------------------------------------------------------------------
      // Check if SPAN PB pressed.
      //------------------------------------------------------------------
      if (Event.reportObject.index == 3 && stack_full_flag==1)    //FORM1                               
        {
          span_pb++;
          if (span_pb ==1)
            {
              genie.WriteObject(GENIE_OBJ_LED, 0x08, 1); //span is set led
              SCNT_LVL_new = glass_level_reading;
              SCNT_LVL_4D_new = abs(glass_level_reading * 1000);
              SCNT_new = beam_count_avg1;
              genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x02, SCNT_new);
              genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x03, SCNT_LVL_4D_new);
            }       
        else
            {
             if (span_pb >1)
                {
                    span_pb=0;
                    genie.WriteObject(GENIE_OBJ_LED, 0x08, 0); //span is set led
                    genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x02, SCNT);
                    genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x03, SCNT_LVL_4D);
                }  
            }
        }
       //------------------------------------------------------------------
      // Check if ENTER PB pressed. (Save Calibration Data)
      //------------------------------------------------------------------
      if (Event.reportObject.index == 11)    //FORM1                               
        {
           if (zero_pb ==1) // new zero?
            {
              ZCNT_LVL = ZCNT_LVL_new;
              ZCNT = ZCNT_new;
              ZCNT_LVL_4D = ZCNT_LVL_4D_new;
              genie.WriteObject(GENIE_OBJ_LED, 0x04, 0); //zero is set led
              Mega_EEPROM_Write_CAL_DATA();
            }   
             
           if (span_pb ==1) // new span?
            {
              SCNT_LVL = SCNT_LVL_new;
              SCNT = SCNT_new;
              SCNT_LVL_4D = SCNT_LVL_4D_new;
              genie.WriteObject(GENIE_OBJ_LED, 0x08, 0); //span is set led
              Mega_EEPROM_Write_CAL_DATA();
            } 
 
            // rebuild stack if cal changed.
            if (zero_pb ==1 || span_pb ==1)
              {
                beam_count_ready=0;
                beam_count=0;
                stack_full_flag=0;  // stack full of readings.
                stack_count=0;
                zero_pb=0;
                span_pb=0;
              }
        } 
 //--------------------------------------------------------------------       
    }
  }
}

//=======================================================================================
// Update Forms
//=======================================================================================
void Update_FORMS()
{
  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x01, glass_level_reading_4D);  // FORM0 Level Reading
  genie.WriteObject(GENIE_OBJ_USER_LED, 0x00, gl_sign);                   // FORM0 Level Reading sign
  genie.WriteObject(GENIE_OBJ_USER_LED, 0x01, wdt_led);                   // FORM0 wdt led 
  wdt_led^=1;
  
  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x03, glass_level_reading_4D);  // FORM2 Level Reading
  genie.WriteObject(GENIE_OBJ_USER_LED, 0x0A, gl_sign);                   // FORM1 Level Reading sign
//  genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x0A, timer1_counter_index);     // FORM4
 // genie.WriteObject(GENIE_OBJ_CUSTOM_DIGITS, 0x0B, SCNT_LVL_4D);     // FORM4
 
  Check_Alarms();
}
